import numpy as np
from numpy import linalg as LA
from Bio.PDB import *
import gzip

def get_average_coord(pdb_file=None, pdb_chain="A", pdb_id=""):
    """
    input:
        pdb_file:file of pdb 
    output
         seq and coordinate  of a pdb file.
    """
    amino2abv = {'ALA':'A',    'ARG':'R',  'ASN':'N',  'ASP':'D', 'CYS':'C',
              'GLN':'Q',    'GLU':'E',  'GLY':'G',  'HIS':'H',  'ILE':'I',
              'LEU':'L',    'LYS':'K',  'MET':'M',  'PHE':'F',  'PRO':'P',
              'SER':'S',    'THR':'T',  'TRP':'W',  'TYR':'Y',  'VAL':'V'}
    if pdb_file[-2:] == 'gz':
        with gzip.open(pdb_file,'rt') as f:
            structure = PDBParser().get_structure(pdb_id,f)
    else:
        structure = PDBParser().get_structure(pdb_id,pdb_file)

    chain = [j for i in list(structure[0]) if i.id == pdb_chain for j in list(i)]
    seq = ''.join([amino2abv[i.resname] for i in chain])
    coord = np.array([np.mean([j.coord for j in i],axis=0) for i in chain])
    return seq, coord


def get_Ca_coord(pdb_file=None, pdb_chain="A", pdb_id=""):
    """
    input:
        pdb_file:file of pdb 
    output
         seq and coordinate  of a pdb file.
    """
    amino2abv = {'ALA':'A',    'ARG':'R',  'ASN':'N',  'ASP':'D', 'CYS':'C',
              'GLN':'Q',    'GLU':'E',  'GLY':'G',  'HIS':'H',  'ILE':'I',
              'LEU':'L',    'LYS':'K',  'MET':'M',  'PHE':'F',  'PRO':'P',
              'SER':'S',    'THR':'T',  'TRP':'W',  'TYR':'Y',  'VAL':'V'}
    if pdb_file[-2:] == 'gz':
        with gzip.open(pdb_file,'rt') as f:
            structure = PDBParser().get_structure(pdb_id,f)
    else:
        structure = PDBParser().get_structure(pdb_id,pdb_file)

    chain = [j for i in list(structure[0]) if i.id == pdb_chain for j in list(i)]
    seq = ''.join([amino2abv[i.resname] for i in chain])
    coord = np.array([j.coord for i in chain for j in i if j.name=="CA"])
    return seq, coord

def calcute_adjacency(coord, threshold=9.5, max_length=None):
    """
    fetch normal adjacency of protein
    chain_amino_coord => distance_matrix => adjacency_matrix => normal_adjacency_matrix
    """
    distance_matrix =  LA.norm(np.array([i-coord  for i in coord]),axis=2)
    adjacency_matrix = np.array(distance_matrix < threshold, dtype=int)
    return adjacency_matrix

def get_Ca_distance(pdb_file,chain="A",threshold=12.5):
    seq,coord = get_Ca_coord(pdb_file,chain=chain)
    Ca_distance = calcute_adjacency(coord,threshold)
    return Ca_distance






import biotite.structure.io.pdb as pdb
import biotite.structure as struc
import gzip

def get_sasa(pdb_file_path):
    """
    pdb_file_path: example  a.pdb or a.pdb.gz
    """
    if pdb_file_path[-3:] == ".gz":
        with gzip.open(pdb_file_path,'rt') as f:
            model = pdb.PDBFile.read(f).get_structure(model=1)
    else:
        with open(pdb_file_path,'r') as f:
            model = pdb.PDBFile.read(f).get_structure(model=1)

    atom_sasa = struc.sasa(model,vdw_radii="Single")    
    res_sasa = struc.apply_residue_wise(model, atom_sasa, np.sum)
    return res_sasa








import numpy as np
from sklearn import metrics
from matplotlib.ticker import MultipleLocator
import matplotlib.pyplot as plt
#import pandas as pd
# (1) scores


def multi_scores(y_true: int, y_pred: float, threshold=0.5, show=False, show_index=True, abbr=True, threshold_soft=False):
    """
    y_true:true label
    y_prob:pred label with probility
    threshold:Negative if y_pred < threshold, positive if y_pred > threshold. 
    threshold_soft:compute a threshold,threshold=(max(y_pred)-min(y_pred))/2

    multi scores of binnary class:
    (1) first layer score
        TP : true positive
        TN : true negative
        FP : false positive
        FN : false engative

    (2) second layer score
        precision   = TP/(TP+FP)
        recall      = TP/(TP+FN)   
        sensitivity = TP/(TP+FN)   
        specificity = TN/(TN+FP)
        Accuracy    = (TP+TN)/(TP+TN+FP+FN)  

    (3) third layer score
         mcc = (TP*TN - FP*FN)/sqrt((TP+FP)*(TP+FN)*(TN+FP)*(TN+FN))
         f1 = 2*(precision*recall)/(precision+recall)

    (4) area score
        auc : Area under the curve of ROC(receipt operator curve) 
        auprc:Area under the precision recall curve
        ap:   Average precision-recall score

    more info:
        TPR = TP/(TP+FN) :true positive rate 
        TNR = TN/(TN+FP) :true negative rate
        FPR = FP/(TN+FP) :false positive rate
        FNR = FN/(TP+FN) :false negative rate

        PPV = TP/(TP+FP):positive predictive value
        NPV = TN/(TN+FN):negative predictive value

        PPV=precision
        TPR=Recall=sensitivity
        TNR=specificity

    example:

    test = scores(
            y_true = [0,   0,   0,   0,   0,   0,   1,   1,   1,   1,   1,   1  ],
            y_pred = [0., 0.2, 0.4, 0.6, 0.8, 1., 0., 0.2, 0.4, 0.6, 0.8, 1],show=True)

    """
    if threshold_soft == True:
        threshold = (max(y_pred)-min(y_pred))/2
    y_true = np.array(y_true, float).ravel()
    y_pred = np.array(y_pred, float).ravel()

    if max(y_true) > (1.+1e-8) or min(y_true) < 0.:
        raise Exception("label not in range (0, 1)!")

    if max(y_pred) > (1.+1e-8) or min(y_pred) < 0.:
        raise Exception("y_prob value not in range (0, 1)!")

    y_true_label = np.round(y_true)
    y_pred_label = np.round(y_pred)

    TP = sum((y_true > threshold) & (y_pred > threshold))
    TN = sum((y_true <= threshold) & (y_pred <= threshold))
    FP = sum((y_true <= threshold) & (y_pred > threshold))
    FN = sum((y_true > threshold) & (y_pred <= threshold))

    #precision =     np.round(metrics.precision_score(y_true_label, y_pred_label),5)
    #recall =        np.round(metrics.recall_score(y_true_label, y_pred_label),5)
    #specificity =   np.round(TN/(TN+FP+1e-6),5)
    if threshold_soft == True:
        PPV = TP/(TP+FP)
        TPR = TP/(TP+FN)
        TNR = TN/(TN+FP)
        precision = PPV
        recall = TPR
        acc = (TP+TN)/(TP+TN+FP+FN)
        mcc = (TP*TN - FP*FN)/np.sqrt((TP+FP)*(TP+FN)*(TN+FP)*(TN+FN))
        f1 = 2*(precision*recall)/(precision+recall)  # or F_beta score
    else:
        import warnings
        warnings.filterwarnings('ignore')
        PPV = np.round(metrics.precision_score(y_true_label, y_pred_label), 5)
        TPR = np.round(metrics.recall_score(y_true_label, y_pred_label), 5)
        TNR = np.round(TN/(TN+FP+1e-6), 5)

        acc = np.round(metrics.accuracy_score(y_true_label, y_pred_label), 5)
        mcc = np.round(metrics.matthews_corrcoef(
            y_true_label, y_pred_label), 5)
        f1 = np.round(metrics.f1_score(y_true_label, y_pred_label), 5)
        warnings.filterwarnings('always')

    precisions, recalls, _ = metrics.precision_recall_curve(y_true, y_pred)
    auroc = np.round(metrics.roc_auc_score(y_true, y_pred), 5)
    auprc = np.round(metrics.auc(recalls, precisions), 5)
    ap = np.round(metrics.average_precision_score(y_true, y_pred), 5)

    scores = (TP, TN, FP, FN, PPV, TPR, TNR, acc, mcc, f1, auroc, auprc, ap)

    np.set_printoptions(suppress=True)
    if show:
        if not abbr and show_index:
            print(
                "TP\tTN\tFP\tFN\tprecision\trecall\tspecificity\taccuracy\tmcc\tf1-score\tAUROC\tAUPRC\tAP")
        elif abbr and show_index:
            print("TP\tTN\tFP\tFN\tPPV\tTPR\tTNR\tAcc\tmcc\tf1\tAUROC\tAUPRC\tAP")
        print("\t".join([str(_) for _ in scores[:5]]) + "\t" +
              "\t".join([str(f"{_:.3f}") for _ in scores[5:]]))
    return scores


def mean_accuray(y_true, y_pred):
    y_true = np.array(y_true, float)
    y_pred = np.array(y_pred, float)

    if max(y_true) > 1 or min(y_true) < 0:
        raise Exception("label not in range (0, 1)!")

    if max(y_pred) > 1 or min(y_pred) < 0:
        raise Exception("y_prob value not in range (0, 1)!")

    y_true_label = np.round(y_true)
    y_pred_label = np.round(y_pred)
    accuracy = np.round(metrics.accuracy_score(y_true_label, y_pred_label), 5)
    return accuracy

##########################
# (1) plot curve of roc and prc


def roc_curve(data_list, labels=None, title=None, colors=None, save_file=False):
    """
    data_list:list of k array with shape(n_example,2) or two columns(y_trues,y_preds).
    """
    # empty plot
    _, ax = plt.subplots(1, 1, figsize=(8, 6))
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.xaxis.set_major_locator(MultipleLocator(0.2))
    ax.set_xlim(0, 1)
    ax.set_ylim(0, 1)
    ax.tick_params(labelsize=18)
    ax.set_xlabel("FPR", fontdict={"fontsize": 18})
    ax.set_ylabel("TPR", fontdict={"fontsize": 18})
    ax.set_title(title if title else "ROC Curve",
                 fontdict={"fontsize": 15}, y=1.05)

    # compute
    rank = {score: index for index, score in enumerate(
        [metrics.roc_auc_score(i[:, 0], i[:, 1]) for i in data_list])}
    for score, index_i in sorted(rank.items(), reverse=True):
        y_true, y_pred = data_list[index_i][:, 0], data_list[index_i][:, 1]
        auc_score = metrics.roc_auc_score(y_true, y_pred)
        fpr, tpr, _ = metrics.roc_curve(y_true, y_pred)
        ax.plot(fpr, tpr,
                label=f"{labels[index_i]}:{auc_score:.3f}" if labels else f"{auc_score:.3f}",
                color=colors[index_i] if colors else None,
                linewidth=2.0)
    plt.legend(fontsize=15, shadow=False, framealpha=0)
    plt.savefig(save_file, dpi=600) if save_file else None
    plt.show()


def pr_curve(data_list, labels=None, title=None, colors=None, save_file=False):
    """
    data_list:list of k array with shape(n_example,2) or two columns(y_trues,y_preds).
    """
    # empty plot
    _, ax = plt.subplots(1, 1, figsize=(8, 6))
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.xaxis.set_major_locator(MultipleLocator(0.2))
    ax.set_xlim(0, 1)
    ax.set_ylim(0, 1)
    ax.tick_params(labelsize=18)
    ax.set_xlabel("Recall", fontdict={"fontsize": 18})
    ax.set_ylabel("Precision", fontdict={"fontsize": 18})
    ax.set_title(label=title if title else "PR Curve",
                 fontdict={"fontsize": 15}, y=1.05)

    # compute
    def auprc(y_true, y_pred):
        precision, recall, _ = metrics.precision_recall_curve(y_true, y_pred)
        return metrics.auc(recall, precision)

    rank = {score: index for index, score in enumerate(
        [auprc(i[:, 0], i[:, 1]) for i in data_list])}
    for score, index_i in sorted(rank.items(), reverse=True):
        y_true, y_pred = data_list[index_i][:, 0], data_list[index_i][:, 1]
        precision, recall, _ = metrics.precision_recall_curve(y_true, y_pred)
        auprc_socre = metrics.auc(recall, precision)

        ax.plot(recall,
                precision,
                label=f"{labels[index_i]}:{auprc_socre:.3f}" if labels else f"{auprc_socre:.3f}",
                color=colors[index_i] if colors else None,
                linewidth=2.0)

    plt.legend(fontsize=15, shadow=False, framealpha=0)
    plt.savefig(save_file, dpi=600) if save_file else None
    plt.show()


def roc_curve_kfold(data_list, labels=None, title=None, colors=None, save_file=False, alpha=0.1, std_show=False):
    """
    data_list:list of fold_list that contain k array with shape(n_example,2) or two column(y_trues,y_pred)
    """
    # empty plot
    _, ax = plt.subplots(1, 1, figsize=(8, 6))
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.xaxis.set_major_locator(MultipleLocator(0.2))
    ax.set_xlim(0, 1)
    ax.set_ylim(0, 1)
    ax.tick_params(labelsize=18)
    ax.set_xlabel("FPR", fontdict={"fontsize": 18})
    ax.set_ylabel("TPR", fontdict={"fontsize": 18})
    ax.set_title(label=title if title else "ROC Curve",
                 fontdict={"fontsize": 15}, y=1.05)
    default_color = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728',
                     '#9467bd', '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']

    # (1) rank
    score_index_ranked = sorted(zip([np.mean([metrics.roc_auc_score(
        j[:, 0], j[:, 1])for j in i]) for i in data_list], np.arange(len(data_list))), reverse=True)
    index_ranked = [i[1] for i in score_index_ranked]

    for rank_i in index_ranked:
        fold_list = data_list[rank_i]

        auroc_score_list = []
        fpr_list = []
        tpr_list = []
        for sub_arr in fold_list:
            y_true = sub_arr[:, 0]
            y_pred = sub_arr[:, 1]
            fpr, tpr, _ = metrics.roc_curve(y_true, y_pred)
            auroc_score = metrics.roc_auc_score(y_true, y_pred)
            fpr_list.append(fpr)
            tpr_list.append(tpr)
            auroc_score_list.append(auroc_score)

        x_tick = fpr_list[np.argmax([len(i) for i in fpr_list])]
        y_ticks = np.zeros((len(fold_list), len(x_tick)))
        for i in range(len(fold_list)):
            for idx_j, j in enumerate(x_tick):
                # insert_value(j,fpr_list[i], tpr_list[i])
                y_ticks[i][idx_j] = np.interp(j, fpr_list[i], tpr_list[i])

        # plot
        if std_show:
            if labels:
                ax_label = f"{labels[rank_i]} {np.mean(auroc_score_list):.3f}±{np.std(auroc_score_list):.3f}"
            else:
                ax_label = f"{np.mean(auroc_score_list):.3f}±{np.std(auroc_score_list):.3f}",
        else:
            if labels:
                ax_label = f"{labels[rank_i]} {np.mean(auroc_score_list):.3f}"
            else:
                ax_label = f"{np.mean(auroc_score_list):.3f}"

        ax.plot(x_tick,
                y_ticks.mean(0),
                label=ax_label,
                color=colors[rank_i] if colors else default_color[rank_i])

        ax.fill_between(x_tick,
                        y_ticks.mean(0)+y_ticks.std(0),
                        y_ticks.mean(0)-y_ticks.std(0),
                        facecolor=colors[rank_i] if colors else default_color[rank_i],
                        alpha=alpha)

    plt.legend(fontsize=15, shadow=False, framealpha=0)
    plt.savefig(save_file, dpi=600) if save_file else None
    # plt.show()
    return ax


def pr_curve_kfold(data_list, 
                    labels=None, 
                    title=None, 
                    colors=None, 
                    save_file=False, 
                    alpha=0.0, 
                    std_show=False,
                    bbox_to_anchor=None):
    """
    data_list:list of fold_list that contain k array with shape(n_example,2) or two column(y_trues,y_pred)
    """
    # empty plot
    _, ax = plt.subplots(1, 1, figsize=(8, 6))
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.xaxis.set_major_locator(MultipleLocator(0.2))
    ax.set_xlim(0, 1)
    ax.set_ylim(0, 1)
    ax.tick_params(labelsize=18)
    ax.set_xlabel("Recall", fontdict={"fontsize": 18})
    ax.set_ylabel("Precision", fontdict={"fontsize": 18})
    ax.set_title(label=title if title else "PR Curve",
                 fontdict={"fontsize": 15}, y=1.05)
    default_color = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728',
                     '#9467bd', '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']

    # compute
    def auprc(ta):
        ap = np.round(metrics.average_precision_score(ta[:,0], ta[:,1]), 5)
        return ap
    
    # (1) rank
    score_index_ranked = sorted(zip([np.mean([auprc(j) for j in i])
                                for i in data_list], np.arange(len(data_list))), reverse=True)
    index_ranked = [i[1] for i in score_index_ranked]

    # (3) compute y lim of precision
    for rank_i in index_ranked:

        fold_list = data_list[rank_i]
        auprc_score_list = []
        precision_list = []
        recall_list = []

        for sub_arr in fold_list:
            y_true = sub_arr[:, 0]
            y_pred = sub_arr[:, 1]
            precision, recall, _ = metrics.precision_recall_curve(
                y_true, y_pred)
            auprc_score = metrics.auc(recall, precision)

            precision_list.append(precision)
            recall_list.append(recall)
            auprc_score_list.append(auprc_score)

        x_tick = recall_list[np.argmax([len(i) for i in precision_list])]
        y_ticks = np.zeros((len(fold_list), len(x_tick)))
        for i_fold in range(len(fold_list)):
            for idx_j, j in enumerate(x_tick):
                if idx_j == 0.:
                    y_ticks[i_fold][idx_j] = 0
                elif idx_j == len(x_tick)-1.:
                    y_ticks[i_fold][idx_j] = 1.
                else:
                    y_ticks[i_fold][idx_j] = np.interp(
                        j, precision_list[i_fold], recall_list[i_fold])
        # plot
        if std_show:
            if labels:
                ax_label = f"{labels[rank_i]} {np.mean(auprc_score_list):.3f}±{np.std(auprc_score_list):.3f}"
            else:
                ax_label = f"{np.mean(auprc_score_list):.3f}±{np.std(auprc_score_list):.3f}",
        else:
            if labels:
                ax_label = f"{labels[rank_i]} {np.mean(auprc_score_list):.3f}"
            else:
                ax_label = f"{np.mean(auprc_score_list):.3f}"

        ax.plot(x_tick,
                y_ticks.mean(0),
                label=ax_label,
                color=colors[rank_i-1] if colors else default_color[rank_i])

        ax.fill_between(x_tick,
                        y_ticks.mean(0)+y_ticks.std(0),
                        y_ticks.mean(0)-y_ticks.std(0),
                        facecolor=colors[rank_i -
                                         1] if colors else default_color[rank_i],
                        alpha=alpha)

    plt.legend(fontsize=13, shadow=False, framealpha=0, bbox_to_anchor=bbox_to_anchor)
    plt.savefig(save_file, dpi=600) if save_file else None
    # plt.show()
    return ax


if __name__ == "__main__":
    test = multi_scores(
        [0,   0,   0,   0,   0,   0,   1,   1,   1,   1,   1,   1],
        [0.001, 0.2, 0.4, 0.6, 0.8, 0.999, 0.001, 0.2, 0.4, 0.6, 0.8, 0.999], show=True)
    #TP,TN,FP,FN, precision, recall, sensitivity, specificity, accuracy, mcc, f1, auroc, auprc,ap = test
    #print(f"TP, TN, FP, FN: {TP}\t{TN}\t{FP}\t{FN}")
    #print(f"precision, recall,sensitivity, specificity: {precision}\t{recall}\t{sensitivity}\t{specificity}")
    #print(f"accuracy, mcc, f1: {accuracy}\t{mcc}\t{f1}" )
    #print(f"auroc, auprc, ap: {auroc}\t{auprc}\t{ap}")



import biotite.structure.io.pdb as pdb
import biotite.application.dssp as dssp
import gzip

def get_sec(pdb_file_path):
    """
    pdb_file_path: example  a.pdb or a.pdb.gz
    return: a numpy array of string

    example:
        from tempfile import gettempdir, NamedTemporaryFile
        import biotite.database.rcsb as rcsb
        pdb_file_path = rcsb.fetch("1l2y", "pdb", gettempdir())
        print(get_sec(pdb_file_path))
    """
    if pdb_file_path[-3:] == ".gz":
        with gzip.open(pdb_file_path,'rt') as f:
            model = pdb.PDBFile.read(f).get_structure(model=1)
    else:
        with open(pdb_file_path,'r') as f:
            model = pdb.PDBFile.read(f).get_structure(model=1)
    app = dssp.DsspApp(model)
    try:
        app.start()
    except:
        raise Exception("Error! you many not install dssp!\ntry 'conda install -c salilab dssp'")
    app.join()
    sse = app.get_sse()
    return sse

if __name__ == "__main__":
    from tempfile import gettempdir, NamedTemporaryFile
    import biotite.database.rcsb as rcsb
    pdb_file_path = rcsb.fetch("1l2y", "pdb", gettempdir())
    print(get_sec(pdb_file_path))




import numpy as np

def dpc(seq):
    aminos = "AGVCDEFILPHNQWKRMSTY"
    dpc_dict = {i+j:0 for i in aminos for j in aminos}
    #seq = seq.replace("X","")
    seq = "".join([i for i in seq if i in aminos])

    for i in range(len(seq)-1):
        dpc_dict[seq[i:i+2]] += 1
    values = np.array(list(dpc_dict.values())) / (len(seq)-1)
    return values





import matplotlib.pyplot as plt
from matplotlib.ticker import MultipleLocator
import numpy as np
from sklearn import metrics


def auprc_curve(y_true, y_pred,save_file=False,label="",title="",color=False,*argvs):
    """
    y_true: true label
    y_pred: pred probility
    
    output: auprc curve
        x-axil :recall  
        y-axil :precision
    """
    _, ax = plt.subplots(1, 1, figsize=(8, 6))
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)

    #axises
    ax.xaxis.set_major_locator(MultipleLocator(0.2))
    ax.set_xlim(0, 1)
    ax.set_ylim(0, 1)
    ax.tick_params(labelsize=26)

    #plot
    AUPRC = round(metrics.average_precision_score(y_true, y_pred), 5)
    precision, recall, threshold = metrics.precision_recall_curve(y_true, y_pred)
    print("x:",recall)
    print("y:",precision )
    print("threshold",threshold)
    if color:
        ax.plot(recall, precision, label=label , color=color, linewidth=2.0)
    else:
        ax.plot(recall, precision, label=label , linewidth=2.0)

    #title and label
    ax.set_xlabel("Recall=TP/(TP+FN)", fontdict={"fontsize": 22})
    ax.set_ylabel("Precision = TP/(TP+FP)", fontdict={"fontsize": 22})
    ax.set_title(title, fontdict={"fontsize": 15}, y=1.05)
    
    return AUPRC


if __name__ == "__main__":
    y_true = [0,   0,   0,   0,   0,   0,   1,   1,   1,   1,   1,   1  ]
    y_pred = [0.0+1e-6, 0.2, 0.4, 0.6, 0.8, 1.0-1e-6, 0.0+1e-6, 0.2, 0.4, 0.6, 0.8, 1.0-1e-6]
    auprc = auprc_curve(y_true,y_pred)
    print(f"auprc:{auprc}")
    plt.show()




"""
generate protein structure feature for GCN.
features:
    adjacency_matrix:we replace adjcency matrix  with revsered distance which turn out work better
    onehot encode for sequece:
    secondary structure of protein:
    solvent accessible surface area of protein
    amino partial charge
    dihedral angles of a protein structure
    hydrogen donor
    dydorgen acceptor
    mass of amino acid
    max solvent accessible surface area of amino acid
    pka1 of amino acid
    pka2 of aminoa acid   
"""
import numpy as np
from numpy import linalg as LA
from Bio.PDB import *
import scipy.sparse as sp
from Bio.PDB.DSSP import DSSP

import gzip
import biotite.structure.io.pdb as pdb
import biotite.structure as struc
import biotite.application.dssp as dssp

def get_rdist(coord, threshold=9.5, max_length=None):
    """  normal adjacency of protein """
    distance_matrix =  LA.norm(np.array([i-coord  for i in coord]),axis=2)
    d = distance_matrix + sp.eye(distance_matrix.shape[0],dtype=np.float32)
    rd = np.power(d,-1)
    rd[np.isinf(d)] = 0
    rd = np.multiply(rd,rd>1/threshold)
    rd = np.power(d,2)
    #rd = sp.csr_matrix(rd) 
    return rd


def pdb2graph_surface_rd2(pdb_file,threshold=9.5,sasa_mask=0.01):
    """

    """
    amino2abv = {
        'ALA':'A',    'ARG':'R',  'ASN':'N',  'ASP':'D',  'CYS':'C',
        'GLN':'Q',    'GLU':'E',  'GLY':'G',  'HIS':'H',  'ILE':'I',
        'LEU':'L',    'LYS':'K',  'MET':'M',  'PHE':'F',  'PRO':'P',
        'SER':'S',    'THR':'T',  'TRP':'W',  'TYR':'Y',  'VAL':'V' }
    amino2mass = {
        "A": 89.1,   "R": 174.2,    "N": 132.12,    "D": 133.11,    "C": 121.16,
        "Q": 146.15, "E": 147.13,   "G": 75.07,     "H": 155.16,    "I": 131.18,
        "L": 131.18, "K": 146.19,   "M": 149.21,    "F": 165.19,    "P": 115.13,
        "S": 105.09, "T": 119.12,   "W": 204.23,    "Y": 181.19,    "V": 117.15, }
    amino2MaxASA={
        "A":106,    "R":248,    "N":157,    "D":163,    "C":135,
        "E":194,    "Q":198,    "G":84,     "H":184,    "I":169,
        "L":164,    "K":205,    "M":188,    "F":197,    "P":136,
        "S":130,    "T":142,    "W":227,    "Y":222,    "V":142,}

    amino2pka1={
        "A":2.34,   "R":2.17,   "N":2.02,   "D":1.88,   "C":1.96,
        "E":2.19,   "Q":2.17,   "G":2.34,   "H":1.82,   "I":2.36,
        "L":2.36,   "K":2.18,   "M":2.28,   "F":1.83,   "P":1.99,
        "S":2.21,   "T":2.09,   "W":2.83,   "Y":2.2,    "V":2.32 }#https://www.chem.ucalgary.ca/courses/351/Carey5th/Ch27/ch27-1-4-2.html
    amino2pka2={
        "A":9.69,   "R":9.04,   "N":8.8,    "D":9.6,    "C":8.18,
        "E":9.67,   "Q":9.13,   "G":9.6,    "H":9.17,   "I":9.6,
        "L":9.6,    "K":8.95,   "M":9.21,   "F":9.13,   "P":10.6,
        "S":9.15,   "T":9.1,    "W":9.39,   "Y":9.11,   "V":9.62,}#https://www.chem.ucalgary.ca/courses/351/Carey5th/Ch27/ch27-1-4-2.html
    amino2Hydropathy ={
        "A":1.8,    "C":2.5,    "D":-3.5,   "E":-3.5,   "F":2.8,
        "G":-0.4,   "H":-3.2,   "I":4.5,    "K":-3.9,   "L":3.8,
        "M":1.9,    "N":-3.5,   "P":-1.6,   "Q":-3.5,   "R":-4.5,
        "S":-0.8,   "T":-0.7,   "V":4.2,    "W":-0.9,   "Y":-1.3}#https://www.cgl.ucsf.edu/chimera/docs/UsersGuide/midas/hydrophob.html; A simple method for displaying the hydropathic character of a protein. K;
    amino2volume={
        "A":88.6,   "R":173.4,  "N":114.1,  "D":111.1,  "C":108.5,
        "Q":143.8,  "E":138.4,  "G":60.1,   "H":153.2,  "I":166.7,
        "L":166.7,  "K":168.6,  "M":162.9,  "F":189.9,  "P":112.7,
        "S":89,     "T":116.1,  "W":227.8,  "Y":193.6,  "V":140, }#https://www.imgt.org/IMGTeducation/Aide-memoire/_UK/aminoacids/abbreviation.html
    #amino2polarity={
    #"A":-0.591, "R":1.538,  "N":0.945,  "D":1.05,   "C":-1.343,
    #"Q":0.931,  "E":1.357,  "G":-0.384, "H":0.336,  "I":-1.239,
    #"L":-1.019, "K":1.831,  "M":-0.663, "F":-1.006, "P":0.189,
    #"S":-0.228, "T":-0.032, "W":-0.595, "Y":0.26,   "V":-1.337}

    amino2num = {i:idx for idx,i in enumerate('ARNDCQEGHILKMFPSTWYV')}
    sec2num = {i:idx for idx,i in enumerate("BCEGHIST")}

    #(1)compute reverse distance matrix
    if pdb_file[-3:] == ".gz":
        with gzip.open(pdb_file,'rt') as f:
            model = pdb.PDBFile.read(f).get_structure(model=1)
    else:
        with open(pdb_file,'r') as f:
            model = pdb.PDBFile.read(f).get_structure(model=1)

    res = "".join([amino2abv[i.res_name] for i in model if i.atom_name=="CA"])   #sequence of a protein.
    coord = [i.coord for i in model if i.atom_name=="CA"]               #C alpha atom coord of a protein.
    rd = get_rdist(coord,threshold) #shape(n,n)(np.float32)
    
    #(2)seq onehot 
    seq_arr = [amino2num[_] for _ in res]
    res_onehot = np.eye(20,dtype=np.float32)[seq_arr]#shape(n,20)(np.float32)
    

    #(3)computer secondary structure
    sec = dssp.DsspApp.annotate_sse(model)  #this function have bug that sometime not work very well!!!
    if len(sec) != len(coord):              #when meet bug,we try biopython method recompute secondary structure.
        sec = ""
        if pdb_file[-3:] == ".gz":
            with gzip.open(pdb_file,"rt") as f:
                structure2 = PDBParser().get_structure("",f)
        else:
            structure2 = PDBParser().get_structure("",pdb_file)
        model2 = structure2[0]
        bio_dssp = DSSP(model2, pdb_file)
        for z in range(len(bio_dssp)):
            a_key = list(bio_dssp.keys())[z]
            sec += bio_dssp[a_key][2]
            sec = sec.replace("-","C")

    sec=np.array([sec2num[_] for _ in sec])
    res_sec_structure = np.eye(8,dtype=np.float32)[sec]#shape(n,1)(np.float32)

    #(4)compute solvent accessible surface area
    atom_sasa = struc.sasa(model,vdw_radii="Single")
    res_sasa = struc.apply_residue_wise(model, atom_sasa, np.sum)/250
    res_sasa = res_sasa.reshape(-1,1)#shape(n,1)(np.float32)

    #(5)comput relative accessible surface area
    #res_rsa = struc.apply_residue_wise(model, atom_sasa, np.sum)/np.array([amino2MaxASA[_] for _ in res],dtype=np.float32)
    #res_rsa = res_rsa.reshape(-1,1)

    #(7)compute the dihedral angles of a protein structure
    #phi,psi,omega = struc.dihedral_backbone(model)
    #res_dihedral_angles = np.stack([np.rad2deg(phi), np.rad2deg(psi)]).T/180
    #res_dihedral_angles[np.isnan( res_dihedral_angles)]=0.5 #shape(n,2)(np.float32)
    
    #(6)amino partial charge
    #model.bonds = struc.connect_via_residue_names(model)
    #atom_partial_charge = struc.partial_charges(model)
    #res_partial_charge = struc.apply_residue_wise(model,atom_partial_charge,np.sum)*50
    #res_partial_charge = res_partial_charge.reshape(-1,1)#shape(n,)(np.float32)


    #(8)Hydrogen donor
    #triplets = struc.hbond(model)
    #res_donor = np.zeros(len(res),dtype=np.float32)#shape(n,1)(np.float32)
    #for _ in triplets[:,0]:
    #    res_donor[model[_].res_id-1]+=1
    #res_donor = res_donor.reshape(-1,1)/6

    #(9)Hydorgen acceptor
    #res_acceptor = np.zeros(len(res),dtype=np.float32)#shape(n,1)(np.float32)
    #for _ in triplets[:,2]:
    #    res_acceptor[model[_].res_id-1]+=1
    #res_acceptor = res_acceptor.reshape(-1,1)/6

    #(10)mass
    #res_mass = np.array([amino2mass[_] for _ in res],dtype=np.float32).reshape(-1,1)/256#shape(n,1)(np.float32)

    #(11)MaxASA
    #res_MaxASA = np.array([amino2MaxASA[_] for _ in res],dtype=np.float32).reshape(-1,1)/512#shape(n,1)(np.float32)

    #(11)Pka(COOH)
    #res_pka1 = np.array([amino2pka1[_] for _ in res],dtype=np.float32).reshape(-1,1)/4#shape(n,1)(np.float32)
    
    #(12)PKa(NH3+)
    #res_pka2 = np.array([amino2pka2[_] for _ in res],dtype=np.float32).reshape(-1,1)/16#shape(n,1)(np.float32)
    
    #(13)hydropathy
    #res_hydropathy = np.array([amino2Hydropathy[_] for _ in res],dtype=np.float32).reshape(-1,1)/8#shape(n,1)(np.float32)

    #(14)volume
    #res_volume = np.array([amino2volume[_] for _ in res],dtype=np.float32).reshape(-1,1)/256


    #compress features to sparse matrix
    #res_onehot,res_sasa,res_sec_structure,res_dihedral_angles,res_partial_charge,res_donor,res_acceptor,res_volume
    mask = np.ones(len(res_sasa),dtype=np.float32)
    mask[res_sasa.ravel()<sasa_mask] = 0
    mask = np.diag(mask)
    rd = mask@rd@mask
    rd = sp.coo_matrix(rd)

    feature = sp.coo_matrix(np.hstack([res_onehot, res_sasa,res_sec_structure]))
    return rd,feature,res
    


"""
amino property info 
(1)MaxASA
Xie, Z., Deng, X., and Shu, K. (2020). Prediction of Protein-Protein Interaction Sites Using Convolutional Neural Network and Improved Data Sets. Int J Mol Sci 21, E467. https://doi.org/10.3390/ijms21020467.


"""



import numpy as np

def cksaap(seq,k=3):
    aminos = "AGVCDEFILPHNQWKRMSTY"
    seq = ''.join([i for i in seq if i in aminos])
    ckdpc =  {str(idx)+i+j:0 for idx in range(k) for i in aminos for j in aminos}

    for idx in range(k):
        for i in range(len(seq)-idx-1):
            ckdpc[str(idx)+seq[i]+seq[i+idx+1]]+=1
    
    values = np.array(list(ckdpc.values()))/ (len(seq)-k)
    return values





import torch
import torch.nn as nn
import torch.nn.functional as F

class cnn_block(nn.Module):
    def __init__(self) -> None:
        super().__init__()
        self.conv1 = nn.Conv1d(21, 32, kernel_size=3, stride=1)
        self.norm1 = nn.BatchNorm1d(64)

        self.conv2 = nn.Conv1d(32, 64, kernel_size=3, stride=1)
        self.norm1 = nn.BatchNorm1d(64)

        self.conv3 = nn.Conv1d(64, 128, kernel_size=3, stride=1)
        self.norm1 = nn.BatchNorm1d(64)

        self.conv4 = nn.Conv1d(128, 256, kernel_size=3, stride=1)
        self.norm1 = nn.BatchNorm1d(64)

        self.conv5 = nn.Conv1d(256, 512, kernel_size=3, stride=1)
        self.norm1 = nn.BatchNorm1d(64)
        self.norm1 = nn.BatchNorm1d(32)
        self.norm2 = nn.BatchNorm1d(64)
        self.maxpool = nn.AvgPool1d(4, 4)   
        self.avgpool = nn.AvgPool1d(4, 4)   

    def forward(self,x):
        x = F.relu(self.conv1(x))
        # x = self.norm1(x)
        x = self.avgpool(x)

        x = F.relu(self.conv2(x))
        # x = self.norm2(x)
        x = self.avgpool(x)

        x = F.relu(self.conv3(x))
        x = self.maxpool(x)

        x = F.relu(self.conv4(x))
        x = self.maxpool(x)

        x = F.relu(self.conv5(x))
        x = self.maxpool(x)
        return x

class siamese_cnn(nn.Module):
    def __init__(self) -> None:
        super().__init__()
        self.cnn_block = cnn_block()
        self.fc1 = nn.Linear(1024,128)
        self.fc2 = nn.Linear(128,2)
        self.flatten = nn.Flatten()
        self.softmax = nn.Softmax()
        self.dropout = nn.Dropout(0.5)


    def forward(self,x1,x2):
        x1 = self.cnn_block(x1)
        x1 = self.flatten(x1)
        x2 = self.cnn_block(x2)
        x2 = self.flatten(x2)
        x = torch.cat((x1,x2),1)
        x = F.relu(self.fc1(x))
        x = self.dropout(x)
        x = self.fc2(x)
        return x


def weights_init(m):
    if isinstance(m,(nn.Conv1d,nn.Linear)):
        nn.init.kaiming_normal_(m.weight, mode='fan_out', nonlinearity='relu')
        #nn.init.constant(m.bias,0.0)

    

#class cnn(nn.Module):
#    def __init__(self):
#        super(siamese, self).__init__()
#        self.conv1 = nn.Conv1d(20, 32, 3)
#        self.norm1 = nn.BatchNorm1d(32)
#        self.pool1 = nn.AvgPool1d(4, 4)
#
#        self.conv2 = nn.Conv1d(32, 64, 3)
#        self.norm2 = nn.BatchNorm1d(64)
#        self.pool2 = nn.AvgPool1d(4, 4)
#
#        self.conv3 = nn.Conv1d(64, 128, 3)
#        self.norm3 = nn.BatchNorm1d(128)
#        self.pool3 = nn.AvgPool1d(4, 4)
#
#        self.conv4 = nn.Conv1d(128, 256, 3)
#        self.norm4 = nn.BatchNorm1d(256)
#        self.pool4 = nn.MaxPool1d(4, 4)
#
#        self.conv5 = nn.Conv1d(256, 512, 3)
#        self.norm5 = nn.BatchNorm1d(512)
#        self.pool5 = nn.MaxPool1d(4, 4)
#
#        self.fc1 = nn.Linear(3584, 64)
#        self.fc2 = nn.Linear(64, 2)
#        self.dropout = nn.Dropout(0.2)
#        # self.sig = nn.Softmax()
#
#    def forward(self, x):
#        batch_size = x.shape[0]
#        x = self.pool1(F.relu(self.norm1(self.conv1(x))))
#        x = self.pool2(F.relu(self.norm2(self.conv2(x))))
#        x = self.pool3(F.relu(self.conv3(x)))
#        x = self.pool4(F.relu(self.conv4(x)))
#        x = x.view(batch_size,-1)
#
#        x = F.relu(self.fc1(x))
#        x = self.dropout(x)
#        x = F.softmax(self.fc2(x), dim = 1)
#
#        return x
#



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zzd",
    version="1.0.0",
    author="zzd lab", 
	author_email="1965770446@qq.com",
    description="encode and scores", 
	long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/miderxi/zzd_lib",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)




import numpy as np

def onehot(seq, fix_len=2000):
    amino2num = { i:idx for idx,i in enumerate("AGVCDEFILPHNQWKRMSTYX")}

    if fix_len:
        seq =np.array([amino2num[i] for i in seq[:fix_len]])
        v = np.zeros((fix_len,21))
        v[:len(seq)] = np.eye(21)[seq]
    else:
        seq = np.array([amino2num[i] for i in seq])
        v =  np.eye(21)[seq]
    return v

if __name__ == "__main__":
    seq="MIRRISPGAVPPSLPSAPVASHHHADAPQAGVQPPADTAHHAPRLRPAPPRKRRRGMRSLDGMDDELDATELEEAEA"
    print(onehot(seq), onehot(seq).shape)




"""
test_ppi.txt train_ppi.txt 
encode:ac, dpc, ct, cksaap, ara2vec, dmi2vec, esm ,pdb2dpc

python RF.py -test=test.txt -train=train.txt -encode="ac,ct,cksaap,..." -out=/tmp/rf.txt

"""
import numpy as np
from Bio import SeqIO
from zzd.feature_encode.feature_combine import feature_combine
from sklearn.svm import SVC

def load_data(ppi_file,seqs_file,features):
    """
    pass
    """
    ppis = np.genfromtxt(ppi_file,str)
    X,Y = np.array(ppis[:,:2],str), np.array(ppis[:,2],int)
    seqs = {i.id:str(i.seq) for i in SeqIO.parse(seqs_file,'fasta')}
    encode = feature_combine(features,features,seqs)
    return encode.encode(X),Y

def ppi_svm(test_file,train_file,seqs_file,feature_encode,n_jobs=-1,**argv):
    """
    test_file:for pred
    train_file:fro train
    seqs_file: fasta sequence
    feature: AC,DPC,CT,CKSAAP,ESM,ara2vec,dmi2vec
    """
    x_train, y_train = load_data(train_file,seqs_file,feature_encode)
    x_test, y_test = load_data(test_file,seqs_file,feature_encode)

    model =  SVC(probability=True,class_weight="balanced",verbose=0,C=6.3, gamma=15.8)
    model.fit(x_train,y_train)
    
    y_test_pred = model.predict_proba(x_test)[:, 1]
    return y_test_pred

if __name__ == "__main__":
    test_file = "../../../atppi/B2_train_and_test/p1n10_10folds/test_0.txt"
    train_file = "../../../atppi/B2_train_and_test/p1n10_10folds/train_0.txt"
    seqs_file = "../../../atppi/B1_ppis_and_seqs/ppis_ara_and_eff.fasta"
    pred = ppi_svm(test_file,train_file,seqs_file,feature_encode=['dpc'],pca=False)



import os
import numpy as np
from Bio import SeqIO
from sklearn.ensemble import RandomForestClassifier
from zzd.feature_encode.feature_combine import feature_combine


def load_data(ppi_file, seqs_file, a_features, b_features, data_file_dict):
    """
    ppi_file: tab format file with 3 column : a_id b_id    label.
    seqs_file:fasta format file.
    a_features:list of protein a feature. such as ac,dpc,ct,...
    b_features:list of protein b feature. such as ac,dpc,ct,...
    data_file_dict:dictionary of personal features that key is name and value is a pkl file.
        pkl file represent a dictionary in which key is protein id and values is a 1d numpy array.
    """
    ppis = np.genfromtxt(ppi_file,str)
    X,Y = np.array(ppis[:,:2],str), np.array(ppis[:,2],int)
    seqs = {i.id:str(i.seq) for i in SeqIO.parse(seqs_file,'fasta')}
    encode = feature_combine(a_features,
                    b_features,
                    seqs,
                    data_file_dict)
    return encode.encode(X),Y


def ppi_randomforest(
                train_file, 
                test_file, 
                seqs_file, 
                a_features,
                b_features,
                data_file_dict = dict(),
                n_estimators=200,
                n_jobs=-1, ):
    """
    train_file: tab format ppi train file with three column: a_id b_id and label
    test_file:  tab format ppi test  file with three column: a_id b_id and label

    seqs_file: fasta sequence
    feature_encode: AC,DPC,CT,CKSAAP,...
    data_file_dict:optional, 
    """
    x_train, y_train = load_data(
                        train_file,
                        seqs_file,
                        a_features,
                        b_features,
                        data_file_dict)

    x_test, y_test = load_data(
                        test_file,
                        seqs_file,
                        b_features,
                        b_features,
                        data_file_dict)
    
    model = RandomForestClassifier(
                        n_estimators=n_estimators,
                        n_jobs=n_jobs, 
                        random_state=1)

    model.fit(x_train,y_train)
    
    y_test_pred = model.predict_proba(x_test)[:, 1]
    return y_test_pred


def ppi_randomforest_predict(
            query_file,
            seqs_file,
            a_features,
            b_features,
            model_file = None,
            data_file_dict = dict(),
            n_estimators = 200,
            n_jobs=-1
            ):

    X = np.genfromtxt(query_file,str) 
    seqs = {i.id:str(i.seq) for i in SeqIO.parse(seqs_file,'fasta')}
    encode = feature_combine(a_features , b_features, seqs_file, data_file_dict)
    x = encode.ensemble(X)

    model = None
    y_pred =  model.predict_proba(x)[:, 1]
    return y_pred

    











#import numpy as np
#from zzd.feature_encode.ac import ac
#from zzd.feature_encode.dpc import dpc
#from zzd.feature_encode.cksaap import cksaap
#from zzd.feature_encode.ct import ct
#from zzd.feature_encode.pdb2graph import pdb2graph
#





import tensorflow as tf
from tensorflow.keras import layers
from tensorflow.keras.models import  Model

def cnn(length=2000,chanels=20,dropout_rate=0.5,lr=0.001):
    In = layers.Input(shape=(length,chanels))
    x = layers.Conv1D(filters=32, kernel_size=3, padding='same', input_shape=(length,chanels))(In)
    x = layers.BatchNormalization()(x)
    x = layers.ReLU()(x)
    x = layers.AvgPool1D(pool_size=4, padding='same')(x)

    x = layers.Conv1D(filters=64, kernel_size=3, strides=1, padding='same')(x)
    x = layers.BatchNormalization()(x)
    x = layers.ReLU()(x)
    x = layers.AvgPool1D(pool_size=4, padding='same')(x)

    x = layers.Conv1D(filters=128, kernel_size=3,padding='same')(x)
    x = layers.ReLU()(x)
    x = layers.AvgPool1D(pool_size=4, padding='same')(x)

    x = layers.Conv1D(filters=256, kernel_size=3, padding='same')(x)
    x = layers.ReLU()(x)
    x = layers.MaxPool1D(pool_size=4, strides=4, padding='same')(x)

    x = layers.Conv1D(filters=512, kernel_size=3, padding='same')(x)
    x = layers.ReLU()(x)
    x = layers.MaxPool1D(pool_size=4, strides=4, padding='same')(x)
    x = layers.Flatten()(x)
    convs = Model(In,x)
    In1 = layers.Input(shape=(length, chanels))
    In2 = layers.Input(shape=(length, chanels))
    x1 = convs(In1)
    x2 = convs(In2)
    x_merge = layers.Concatenate()([x1,x2])
    fc1 = layers.Dense(512, activation='relu')(x_merge)
    fc1 = layers.Dropout(dropout_rate)(fc1)
    fc2 = layers.Dense(1, activation='sigmoid')(fc1)
    model = Model([In1,In2],[fc2])

    model.compile(loss = tf.keras.losses.BinaryCrossentropy(),
                optimizer=tf.keras.optimizers.Adam(learning_rate=lr),metrics=['accuracy'])

    return model




"""
example
python interolog ppi seqs

#mechanism
protein A` is the homolog of protein A
protein B` is the homolog of protein B
if A`-B` have interation then we inference A-B also have interation.

#
1. protein sequences db in which protein also have ppi.
2. seach homolog of input protein pair
3. if any interaction of A`-B` then A-B interaction. vice versa.


python -test test_ppis.txt -train train_ppis.txt  -seqs fasta_seq
python -test test_ppis.txt

"""
import sys
import os
import numpy as np
import pandas as pd
from Bio import SeqIO
#import torch.nn as nn


#1 load input ppi and interolog_ppi_db
def enviroment_init():
    """
    download ppidb and ppiblastdb.
    you need install ncbi-blastp program first and add the blastp path to $PATH.
    """
    if not os.path.exists(f"{os.environ['HOME']}/.local/interolog/ppis_db.txt"):
        os.system("mkdir -p $HOME/.local/zzd/interolog/blastdb")
        os.system("wget https://github.com/miderxi/zzd_lib/raw/main/zzd/lib/blastdb.tar.gz \
                    -O $HOME/.local/zzd/interolog/blastdb/blastdb.tar.gz")
        os.system("tar -xvf $HOME/.local/zzd/interolog/blastdb/blastdb.tar.gz \
                    -C $HOME/.local/zzd/interolog/blastdb/")
        os.system("makeblastdb \
                    -in $HOME/.local/zzd/interolog/blastdb/blastdb.fasta \
                    -out $HOME/.local/zzd/interolog/blastdb/blastdb  -dbtype prot")
        os.system("wget https://github.com/miderxi/zzd_lib/raw/main/zzd/lib/ppis_db.tar.gz\
                    -O $HOME/.local/zzd/interolog/ppis_db.tar.gz")
        os.system("tar -xvf $HOME/.local/zzd/interolog/ppis_db.tar.gz \
                    -C $HOME/.local/zzd/interolog/")
 
def fetch_ppis_db(train_file=None,seqs_file=None):
    ppis_db  = set([tuple(i) for i in  
            pd.read_table(f"{os.environ['HOME']}/.local/zzd/interolog/ppis_db.txt",header=None).to_numpy()])
    if train_file:
        temp = np.genfromtxt(train_file,str,delimiter="\t")
        if temp.shape[1] == 3:
            temp = temp[temp[:,2] == "1"]
        for i in temp:
            ppis_db.add(tuple(i))
        if not os.path.existst("/tmp/interolog"):
            os.system("mkdir /tmp/interolog/")
        os.system(f"makeblastdb -in {seqs_file} -out /tmp/interolog/seqs -dbtype prot")
    return ppis_db


def fetch_homo(query_ppis,seqs_file,train_file,threshold=40,runblast=True,n_works=8):
    ids_query =set([j for i in  query_ppis for j in i])
    query_seqs = {i.id:str(i.seq) for i in SeqIO.parse(seqs_file,"fasta") if i.id in ids_query}
    with open("/tmp/interolog/query_seqs.fasta","w") as f :
        for k,v in query_seqs.items():
            f.write(f">{k}\n{v}\n")
    
    cmd1 = f"blastp -query /tmp/interolog/query_seqs.fasta \
                -db {os.environ['HOME']}/.local/interolog/blastdb/blastdb\
                -out /tmp/interolog/query_seqs.blastp \
                -evalue 1e-2 \
                -outfmt 6 \
                -num_threads {n_works}"
    cmd2 = f"blastp -query /tmp/interolog/query_seqs.fasta \
                -db /tmp/interolog/seqs\
                -out /tmp/interolog/query_seqs2.blastp \
                -evalue 1e-2 \
                -outfmt 6 \
                -num_threads {n_works}"
    if runblast:
        os.system(cmd1)
        if train_file:
            os.system(cmd2)
    
    homo = {}
    for line in pd.read_table("/tmp/interolog/query_seqs.blastp",header=None).to_numpy():
        if line[2] > threshold:
            if line[0] not in homo.keys():
                homo[line[0]] = set([line[1]])
            else:
                homo[line[0]].add(line[1])
    
    if train_file:
         for line in pd.read_table(f"/tmp/interolog/query_seqs2.blastp",header=None).to_numpy():
            if line[2] > threshold:
                if line[0] not in homo.keys():
                    homo[line[0]] = set([line[1]])
                else:
                    homo[line[0]].add(line[1])
    return homo



def prediction(query_ppis, ppis_db, homo,detail=True):
    ppis_pred = []
    for a,b in query_ppis[:,:2].reshape(-1,2):
        if detail:
            pred = [a,b,0,None]
        else:
            pred = 0

        if a in homo.keys() and b in homo.keys():
            candicate = set([(i,j) for i in  homo[a] for j in homo[b]] +
                        [(j,i) for i in  homo[a] for j in homo[b]])
            if candicate & ppis_db:
                if detail:
                    pred[2] = 1
                    pred[3] = candicate & ppis_db
                else:
                    pred = 1    
        ppis_pred.append(pred)
    return ppis_pred


def interolog(test_file, seqs_file=None,train_file=None, threshold=40,runblast=True,detail=False,n_works=8):
    """
    test_file:'protein_a\tprotein_b'  pair per line for predict.
    train_file:'protein_a\t protein_b' per line to infer from.   
    seqs_file:fasta format file
    """
    enviroment_init()
    query_ppis = np.genfromtxt(test_file,str,delimiter="\t")
    ppis_db = fetch_ppis_db(train_file=train_file,seqs_file=seqs_file)
    homo = fetch_homo(query_ppis,seqs_file,train_file,threshold,runblast,n_works=8)
    pred = prediction(query_ppis,ppis_db,homo,detail=detail)
    return pred


	
if __name__ == "__main__":
    from zzd import scores
    test_file = "../../../atppi/B2_train_and_test/p1n10_10folds/test_0.txt"
    train_file = "../../../atppi/B2_train_and_test/p1n10_10folds/train_0.txt"
    seqs_file = "../../../atppi/B1_ppis_and_seqs/ppis_ara_and_eff.fasta"

    #pred = interolog(test_file,seqs_file,train_file,runblast=False,detail=True)
    #[print(i) for i in pred]
    pred = interolog(test_file,seqs_file,train_file,runblast=False,detail=False,threshold=10,n_works=8)	
    scores(pd.read_table(test_file,header=None).to_numpy()[:,2],pred,show=True)

